import {ref, computed} from 'vue';
import {acceptHMRUpdate, defineStore} from 'pinia';
import {useAuth} from 'feathers-pinia-v4';
import {useRouter} from 'vue-router';


const $lget = require('lodash/get');

export default (
  {
    useFeathers,
    userService = 'users',
    loginService = 'logins',
    accountService = 'accounts',
    useAuthOptions = {},
  } = {}) => {
  const useAuthStore = defineStore('auth', () => {
    const router = useRouter();
    const {$feathersApi} = useFeathers();

    let payload = ref(null);
    let rules = ref([]);
    const activeLoginId = ref(null);
    const logins = ref([]);
    const activeAccountId = ref(null);

    const activeLogin = computed(() => {
      return activeLoginId.value ? $feathersApi.service(loginService).getFromStore(activeLoginId.value).value : null;
    });
    const accountIds = computed(() => {
      return $lget(activeLogin.value, 'accounts.owns.ids', []);
    });
    const activeAccount = computed(() => {
      return activeAccountId.value ? $feathersApi.service(accountService).getFromStore(activeAccountId.value).value : null;
    });

    const accounts = computed(() => {
      const {data} = $feathersApi.service(accountService).findInStore({
        query: {
          _id: {
            $in: accountIds.value,
          },
        },
      });
      return accountIds.value.length ? data : [];
    });

    const setup = async ({user, ...res}) => {
      rules.value = $lget(user, '_fastjoin.rules', []);
      logins.value = $lget(user, '_fastjoin.logins.ids', []).map(login => {
        return $feathersApi.service(loginService).createInStore(login);
      });
      let login = $lget(user, '_fastjoin.logins.active', logins.value[0] || undefined);
      if (login) activeLoginId.value = login._id;
      $lget(activeLogin.value, '_fastjoin.accounts.owns.ids', []).forEach(account => {
        $feathersApi.service(accountService).createInStore(account);
      });
      let account = $lget(activeLogin.value, '_fastjoin.accounts.owns.active', accounts.value[0] || undefined);
      if (account) activeAccountId.value = account._id;
      return {
        ...res,
        user,
      };
    };
    const handleError = async (error) => {
      // handle the error as you see fit.
      console.log(error);
    };

    const utils = useAuth({
      api: $feathersApi,
      servicePath: userService,
      onSuccess: async (result) => {
        console.log('onSuccess');
        payload.value = result;
        return await setup(result);
      },
      onError: async (error) => {
        console.log('onError', error);
        await handleError(error);
      },
      // eslint-disable-next-line no-unused-vars
      onInitSuccess: async (result) => {
        console.log('onInitSuccess');
      },
      onInitError: async (error) => {
        console.log('onInitError', error);
        await handleError(error);
      },
      // eslint-disable-next-line no-unused-vars
      onLogoutSuccess: async (result) => {
        console.log('onLogoutSuccess');
        // $feathersApi.service(userService).clearAll();
      },
      onLogoutError: async (error) => {
        console.log(error);
      },
      ...useAuthOptions,
    });

    utils.reAuthenticate();

    return {
      ...utils,
      logout: async (redirectPath = '/login') => {
        await utils.logout();
        router.push(redirectPath);
      },

      payload,
      rules,
      activeLoginId,
      activeLogin,
      logins,
      activeAccountId,
      activeAccount,
      accountIds,
      accounts,
    };
  });

  if (import.meta.hot) {
    import.meta.hot.accept(acceptHMRUpdate(useAuthStore, import.meta.hot));
  }

  return useAuthStore;
  // return defineAuthStore({
  //   $feathersApi,
  //   userService,
  //   state() {
  //     return {
  //       userId: null,
  //
  //       rules: [],
  //       logins: [],
  //       activeLoginId: null,
  //       activeAccountId: null,
  //       ...state(),
  //     };
  //   },
  //   getters: {
  //     authUser() {
  //       return this.userId ? Users.getFromStore(this.userId) : null;
  //     },
  //     activeLogin() {
  //       return this.activeLoginId ? Logins.getFromStore(this.activeLoginId) : null;
  //     },
  //     activeAccount() {
  //       return this.activeAccountId ? Accounts.getFromStore(this.activeAccountId) : null;
  //     },
  //     accountIds() {
  //       return $lget(this.activeLogin, 'accounts.owns.ids', []);
  //     },
  //     accounts() {
  //       const {data} = Accounts.findInStore({
  //         query: {
  //           _id: {
  //             $in: this.accountIds
  //           },
  //         },
  //       });
  //       return this.accountIds.length ? data : [];
  //     },
  //     ...getters,
  //   },
  //   actions: {
  //     setActiveAccount(account) {
  //       let model = new models.api.Accounts(account);
  //       model.addToStore();
  //       this.activeAccountId = model._id;
  //     },
  //     handleResponse(response) {
  //       this.payload = response || null;
  //       this.userId = response.user.id || response.user._id;
  //
  //       this.rules = $lget(response.user, '_fastjoin.rules', []);
  //       if (Logins && Accounts) {
  //         this.logins = $lget(response.user, '_fastjoin.logins.ids', []).map(login => {
  //           let model = new models.api.Logins(login);
  //           model.addToStore();
  //           return model;
  //         });
  //         let login = $lget(response.user, '_fastjoin.logins.active', this.logins[0] || undefined);
  //         if (login) this.activeLoginId = login._id;
  //          $lget(this.activeLogin, '_fastjoin.accounts.owns.ids', []).forEach(account => {
  //           let model = new models.api.Accounts(account);
  //           model.addToStore();
  //         });
  //         let account = $lget(this.activeLogin, '_fastjoin.accounts.owns.active', this.accounts[0] || undefined);
  //         if (account) this.activeAccountId = account._id;
  //       }
  //
  //       Users.addToStore(response.user);
  //       return response;
  //     },
  //     logout() {
  //       return this.$feathersApi
  //         .logout()
  //         .then(response => {
  //
  //           this.payload = null;
  //           this.userId = null;
  //           if (this.user) {
  //             this.user = null;
  //           }
  //           return response;
  //         })
  //         .catch(error => {
  //           return Promise.reject(error);
  //         });
  //     },
  //
  //     ...actions,
  //   },
  // });
};
